=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2005-09-06 07:39:09 +0000
+++ bzrlib/branch.py	2005-09-07 04:00:21 +0000
@@ -30,7 +30,7 @@
 from bzrlib.revision import Revision
 from bzrlib.delta import compare_trees
 from bzrlib.tree import EmptyTree, RevisionTree
-import bzrlib.xml
+import bzrlib.xml5
 import bzrlib.ui
 
 
@@ -309,7 +309,8 @@
             "This is a Bazaar-NG control directory.\n"
             "Do not change any files in this directory.\n")
         self.controlfile('branch-format', 'w').write(BZR_BRANCH_FORMAT_5)
-        for d in ('text-store', 'inventory-store', 'revision-store'):
+        for d in ('text-store', 'inventory-store', 'revision-store',
+                  'weaves'):
             os.mkdir(self.controlfilename(d))
         for f in ('revision-history', 'merged-patches',
                   'pending-merged-patches', 'branch-name',
@@ -322,7 +323,7 @@
         # them; they're not needed for now and so ommitted for
         # simplicity.
         f = self.controlfile('inventory','w')
-        bzrlib.xml.serializer_v4.write_inventory(Inventory(), f)
+        bzrlib.xml5.serializer_v5.write_inventory(Inventory(), f)
 
 
     def _check_format(self):
@@ -369,7 +370,7 @@
             # ElementTree does its own conversion from UTF-8, so open in
             # binary.
             f = self.controlfile('inventory', 'rb')
-            return bzrlib.xml.serializer_v4.read_inventory(f)
+            return bzrlib.xml5.serializer_v5.read_inventory(f)
         finally:
             self.unlock()
             
@@ -386,7 +387,7 @@
         try:
             f = AtomicFile(self.controlfilename('inventory'), 'wb')
             try:
-                bzrlib.xml.serializer_v4.write_inventory(inv, f)
+                bzrlib.xml5.serializer_v5.write_inventory(inv, f)
                 f.commit()
             finally:
                 f.close()
@@ -601,7 +602,7 @@
         xml_file = self.get_revision_xml_file(revision_id)
 
         try:
-            r = bzrlib.xml.serializer_v4.read_revision(xml_file)
+            r = bzrlib.xml5.serializer_v5.read_revision(xml_file)
         except SyntaxError, e:
             raise bzrlib.errors.BzrError('failed to unpack revision_xml',
                                          [revision_id,
@@ -654,7 +655,7 @@
         from bzrlib.inventory import Inventory
 
         f = self.get_inventory_xml_file(inventory_id)
-        return bzrlib.xml.serializer_v4.read_inventory(f)
+        return bzrlib.xml5.serializer_v5.read_inventory(f)
 
 
     def get_inventory_xml(self, inventory_id):
@@ -872,8 +873,8 @@
        
 
     def commit(self, *args, **kw):
-        from bzrlib.commit import commit
-        commit(self, *args, **kw)
+        from bzrlib.commit import Commit
+        Commit().commit(self, *args, **kw)
         
 
     def lookup_revision(self, revision):

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2005-09-05 09:27:11 +0000
+++ bzrlib/builtins.py	2005-09-07 04:00:21 +0000
@@ -946,6 +946,8 @@
     aliases = ['ci', 'checkin']
 
     # TODO: Give better message for -s, --summary, used by tla people
+
+    # XXX: verbose currently does nothing
     
     def run(self, message=None, file=None, verbose=True, selected_list=None,
             unchanged=False):
@@ -975,7 +977,7 @@
             message = codecs.open(file, 'rt', bzrlib.user_encoding).read()
 
         try:
-            b.commit(message, verbose=verbose,
+            b.commit(message,
                      specific_files=selected_list,
                      allow_pointless=unchanged)
         except PointlessCommit:

=== modified file 'bzrlib/commit.py'
--- bzrlib/commit.py	2005-09-06 07:54:19 +0000
+++ bzrlib/commit.py	2005-09-07 04:00:21 +0000
@@ -15,6 +15,16 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 
+## XXX: Everything up to here can simply be orphaned if we abort
+## the commit; it will leave junk files behind but that doesn't
+## matter.
+
+## TODO: Read back the just-generated changeset, and make sure it
+## applies and recreates the right state.
+
+
+import os
+import sys
 import time
 import tempfile
 from binascii import hexlify
@@ -22,289 +32,352 @@
 from bzrlib.osutils import (local_time_offset, username,
                             rand_bytes, compact_date, user_email,
                             kind_marker, is_inside_any, quotefn,
-                            sha_string, isdir, isfile)
+                            sha_string, sha_file, isdir, isfile)
 from bzrlib.branch import gen_file_id
 from bzrlib.errors import BzrError, PointlessCommit
 from bzrlib.revision import Revision, RevisionReference
 from bzrlib.trace import mutter, note
-from bzrlib.xml import serializer_v4
+from bzrlib.xml5 import serializer_v5
 from bzrlib.inventory import Inventory
-
-
-def commit(branch, message,
-           timestamp=None,
-           timezone=None,
-           committer=None,
-           verbose=True,
-           specific_files=None,
-           rev_id=None,
-           allow_pointless=True):
-    """Commit working copy as a new revision.
-
-    The basic approach is to add all the file texts into the
-    store, then the inventory, then make a new revision pointing
-    to that inventory and store that.
-
-    This is not quite safe if the working copy changes during the
-    commit; for the moment that is simply not allowed.  A better
-    approach is to make a temporary copy of the files before
-    computing their hashes, and then add those hashes in turn to
-    the inventory.  This should mean at least that there are no
-    broken hash pointers.  There is no way we can get a snapshot
-    of the whole directory at an instant.  This would also have to
-    be robust against files disappearing, moving, etc.  So the
-    whole thing is a bit hard.
-
-    This raises PointlessCommit if there are no changes, no new merges,
-    and allow_pointless  is false.
-
-    timestamp -- if not None, seconds-since-epoch for a
-         postdated/predated commit.
-
-    specific_files
-        If true, commit only those files.
-
-    rev_id
-        If set, use this as the new revision id.
-        Useful for test or import commands that need to tightly
-        control what revisions are assigned.  If you duplicate
-        a revision id that exists elsewhere it is your own fault.
-        If null (default), a time/random revision id is generated.
+from bzrlib.delta import compare_trees
+from bzrlib.weave import Weave
+from bzrlib.weavefile import read_weave, write_weave_v5
+from bzrlib.atomicfile import AtomicFile
+
+
+class NullCommitReporter(object):
+    """I report on progress of a commit."""
+    def added(self, path):
+        pass
+
+    def removed(self, path):
+        pass
+
+    def renamed(self, old_path, new_path):
+        pass
+
+
+class ReportCommitToLog(NullCommitReporter):
+    def added(self, path):
+        note('added %s', path)
+
+    def removed(self, path):
+        note('removed %s', path)
+
+    def renamed(self, old_path, new_path):
+        note('renamed %s => %s', old_path, new_path)
+
+
+class Commit(object):
+    """Task of committing a new revision.
+
+    This is a MethodObject: it accumulates state as the commit is
+    prepared, and then it is discarded.  It doesn't represent
+    historical revisions, just the act of recording a new one.
+
+            missing_ids
+            Modified to hold a list of files that have been deleted from
+            the working directory; these should be removed from the
+            working inventory.
     """
-
-    branch.lock_write()
-
-    try:
-        # First walk over the working inventory; and both update that
-        # and also build a new revision inventory.  The revision
-        # inventory needs to hold the text-id, sha1 and size of the
-        # actual file versions committed in the revision.  (These are
-        # not present in the working inventory.)  We also need to
-        # detect missing/deleted files, and remove them from the
-        # working inventory.
-
-        work_tree = branch.working_tree()
-        work_inv = work_tree.inventory
-        basis = branch.basis_tree()
-        basis_inv = basis.inventory
-
-        if verbose:
-            # note('looking for changes...')
-            # print 'looking for changes...'
-            # disabled; should be done at a higher level
-            pass
-
-        pending_merges = branch.pending_merges()
-
-        missing_ids, new_inv, any_changes = \
-                     _gather_commit(branch,
-                                    work_tree,
-                                    work_inv,
-                                    basis_inv,
-                                    specific_files,
-                                    verbose)
-
-        if not (any_changes or allow_pointless or pending_merges):
-            raise PointlessCommit()
-
-        for file_id in missing_ids:
-            # Any files that have been deleted are now removed from the
-            # working inventory.  Files that were not selected for commit
-            # are left as they were in the working inventory and ommitted
-            # from the revision inventory.
-
-            # have to do this later so we don't mess up the iterator.
-            # since parents may be removed before their children we
-            # have to test.
-
-            # FIXME: There's probably a better way to do this; perhaps
-            # the workingtree should know how to filter itbranch.
-            if work_inv.has_id(file_id):
-                del work_inv[file_id]
-
-        if rev_id is None:
-            rev_id = _gen_revision_id(branch, time.time())
-        inv_id = rev_id
-
+    def __init__(self,
+                 reporter=None):
+        if reporter is not None:
+            self.reporter = reporter
+        else:
+            self.reporter = NullCommitReporter()
+
+        
+    def commit(self,
+               branch, message,
+               timestamp=None,
+               timezone=None,
+               committer=None,
+               specific_files=None,
+               rev_id=None,
+               allow_pointless=True):
+        """Commit working copy as a new revision.
+
+        The basic approach is to add all the file texts into the
+        store, then the inventory, then make a new revision pointing
+        to that inventory and store that.
+
+        This is not quite safe if the working copy changes during the
+        commit; for the moment that is simply not allowed.  A better
+        approach is to make a temporary copy of the files before
+        computing their hashes, and then add those hashes in turn to
+        the inventory.  This should mean at least that there are no
+        broken hash pointers.  There is no way we can get a snapshot
+        of the whole directory at an instant.  This would also have to
+        be robust against files disappearing, moving, etc.  So the
+        whole thing is a bit hard.
+
+        This raises PointlessCommit if there are no changes, no new merges,
+        and allow_pointless  is false.
+
+        timestamp -- if not None, seconds-since-epoch for a
+             postdated/predated commit.
+
+        specific_files
+            If true, commit only those files.
+
+        rev_id
+            If set, use this as the new revision id.
+            Useful for test or import commands that need to tightly
+            control what revisions are assigned.  If you duplicate
+            a revision id that exists elsewhere it is your own fault.
+            If null (default), a time/random revision id is generated.
+        """
+
+        self.branch = branch
+        self.branch.lock_write()
+        self.rev_id = rev_id
+        self.specific_files = specific_files
+
+        if timestamp is None:
+            self.timestamp = time.time()
+        else:
+            self.timestamp = long(timestamp)
+            
+        if committer is None:
+            self.committer = username(self.branch)
+        else:
+            assert isinstance(committer, basestring), type(committer)
+            self.committer = committer
+
+        if timezone is None:
+            self.timezone = local_time_offset()
+        else:
+            self.timezone = int(timezone)
+
+        assert isinstance(message, basestring), type(message)
+        self.message = message
+
+        try:
+            # First walk over the working inventory; and both update that
+            # and also build a new revision inventory.  The revision
+            # inventory needs to hold the text-id, sha1 and size of the
+            # actual file versions committed in the revision.  (These are
+            # not present in the working inventory.)  We also need to
+            # detect missing/deleted files, and remove them from the
+            # working inventory.
+
+            self.work_tree = self.branch.working_tree()
+            self.work_inv = self.work_tree.inventory
+            self.basis_tree = self.branch.basis_tree()
+            self.basis_inv = self.basis_tree.inventory
+
+            self.pending_merges = self.branch.pending_merges()
+
+            if self.rev_id is None:
+                self.rev_id = _gen_revision_id(self.branch, time.time())
+
+            self.delta = compare_trees(self.basis_tree, self.work_tree,
+                                       specific_files=self.specific_files)
+
+            if not (self.delta.has_changed()
+                    or self.allow_pointless
+                    or self.pending_merges):
+                raise PointlessCommit()
+
+            self.new_inv = self.basis_inv.copy()
+
+            self.delta.show(sys.stdout)
+
+            self._remove_deleted()
+            self._store_texts()
+
+            self.branch._write_inventory(self.work_inv)
+            self._record_inventory()
+
+            self._make_revision()
+            note('committted r%d', (self.branch.revno() + 1))
+            self.branch.append_revision(rev_id)
+            self.branch.set_pending_merges([])
+        finally:
+            self.branch.unlock()
+
+
+    def _record_inventory(self):
         inv_tmp = tempfile.TemporaryFile()
-        
-        serializer_v4.write_inventory(new_inv, inv_tmp)
-        inv_tmp.seek(0)
-        branch.inventory_store.add(inv_tmp, inv_id)
-        mutter('new inventory_id is {%s}' % inv_id)
-
-        # We could also just sha hash the inv_tmp file
-        # however, in the case that branch.inventory_store.add()
-        # ever actually does anything special
-        inv_sha1 = branch.get_inventory_sha1(inv_id)
-
-        branch._write_inventory(work_inv)
-
-        if timestamp == None:
-            timestamp = time.time()
-
-        if committer == None:
-            committer = username(branch)
-
-        if timezone == None:
-            timezone = local_time_offset()
-
-        mutter("building commit log message")
-        rev = Revision(timestamp=timestamp,
-                       timezone=timezone,
-                       committer=committer,
-                       message = message,
-                       inventory_id=inv_id,
-                       inventory_sha1=inv_sha1,
-                       revision_id=rev_id)
-
-        rev.parents = []
-        precursor_id = branch.last_patch()
+        serializer_v5.write_inventory(self.new_inv, inv_tmp)
+        inv_tmp.seek(0)
+        self.inv_sha1 = sha_file(inv_tmp)
+        inv_tmp.seek(0)
+        self.branch.inventory_store.add(inv_tmp, self.rev_id)
+
+
+    def _make_revision(self):
+        """Record a new revision object for this commit."""
+        self.rev = Revision(timestamp=self.timestamp,
+                            timezone=self.timezone,
+                            committer=self.committer,
+                            message=self.message,
+                            inventory_sha1=self.inv_sha1,
+                            revision_id=self.rev_id)
+
+        self.rev.parents = []
+        precursor_id = self.branch.last_patch()
         if precursor_id:
-            precursor_sha1 = branch.get_revision_sha1(precursor_id)
-            rev.parents.append(RevisionReference(precursor_id, precursor_sha1))
-        for merge_rev in pending_merges:
-            rev.parents.append(RevisionReference(merge_rev))            
+            self.rev.parents.append(RevisionReference(precursor_id))
+        for merge_rev in self.pending_merges:
+            rev.parents.append(RevisionReference(merge_rev))
 
         rev_tmp = tempfile.TemporaryFile()
-        serializer_v4.write_revision(rev, rev_tmp)
+        serializer_v5.write_revision(self.rev, rev_tmp)
         rev_tmp.seek(0)
-        branch.revision_store.add(rev_tmp, rev_id)
-        mutter("new revision_id is {%s}" % rev_id)
-
-        ## XXX: Everything up to here can simply be orphaned if we abort
-        ## the commit; it will leave junk files behind but that doesn't
-        ## matter.
-
-        ## TODO: Read back the just-generated changeset, and make sure it
-        ## applies and recreates the right state.
-
-        ## TODO: Also calculate and store the inventory SHA1
-        mutter("committing patch r%d" % (branch.revno() + 1))
-
-        branch.append_revision(rev_id)
-
-        branch.set_pending_merges([])
-
-        if verbose:
-            # disabled; should go through logging
-            # note("commited r%d" % branch.revno())
-            # print ("commited r%d" % branch.revno())
-            pass
-    finally:
-        branch.unlock()
-
-
-
-def _gen_revision_id(branch, when):
-    """Return new revision-id."""
-    s = '%s-%s-' % (user_email(branch), compact_date(when))
-    s += hexlify(rand_bytes(8))
-    return s
-
-
-def _gather_commit(branch, work_tree, work_inv, basis_inv, specific_files,
-                   verbose):
-    """Build inventory preparatory to commit.
-
-    Returns missing_ids, new_inv, any_changes.
-
-    This adds any changed files into the text store, and sets their
-    test-id, sha and size in the returned inventory appropriately.
-
-    missing_ids
-        Modified to hold a list of files that have been deleted from
-        the working directory; these should be removed from the
-        working inventory.
-    """
-    any_changes = False
-    inv = Inventory(work_inv.root.file_id)
-    missing_ids = []
-    
-    for path, entry in work_inv.iter_entries():
-        ## TODO: Check that the file kind has not changed from the previous
-        ## revision of this file (if any).
-
-        p = branch.abspath(path)
-        file_id = entry.file_id
-        mutter('commit prep file %s, id %r ' % (p, file_id))
-
-        if specific_files and not is_inside_any(specific_files, path):
-            mutter('  skipping file excluded from commit')
-            if basis_inv.has_id(file_id):
-                # carry over with previous state
-                inv.add(basis_inv[file_id].copy())
-            else:
-                # omit this from committed inventory
-                pass
-            continue
-
-        if not work_tree.has_id(file_id):
-            if verbose:
-                print('deleted %s%s' % (path, kind_marker(entry.kind)))
-                any_changes = True
-            mutter("    file is missing, removing from inventory")
-            missing_ids.append(file_id)
-            continue
-
-        # this is present in the new inventory; may be new, modified or
-        # unchanged.
-        old_ie = basis_inv.has_id(file_id) and basis_inv[file_id]
-        
-        entry = entry.copy()
-        inv.add(entry)
-
-        if old_ie:
-            old_kind = old_ie.kind
-            if old_kind != entry.kind:
-                raise BzrError("entry %r changed kind from %r to %r"
-                        % (file_id, old_kind, entry.kind))
-
-        if entry.kind == 'directory':
-            if not isdir(p):
-                raise BzrError("%s is entered as directory but not a directory"
-                               % quotefn(p))
-        elif entry.kind == 'file':
-            if not isfile(p):
-                raise BzrError("%s is entered as file but is not a file" % quotefn(p))
-
-            new_sha1 = work_tree.get_file_sha1(file_id)
-
-            if (old_ie
-                and old_ie.text_sha1 == new_sha1):
-                ## assert content == basis.get_file(file_id).read()
-                entry.text_id = old_ie.text_id
-                entry.text_sha1 = new_sha1
-                entry.text_size = old_ie.text_size
-                mutter('    unchanged from previous text_id {%s}' %
-                       entry.text_id)
-            else:
-                content = file(p, 'rb').read()
-
-                # calculate the sha again, just in case the file contents
-                # changed since we updated the cache
-                entry.text_sha1 = sha_string(content)
-                entry.text_size = len(content)
-
-                entry.text_id = gen_file_id(entry.name)
-                branch.text_store.add(content, entry.text_id)
-                mutter('    stored with text_id {%s}' % entry.text_id)
-
-        if verbose:
+        self.branch.revision_store.add(rev_tmp, self.rev_id)
+        mutter('new revision_id is {%s}', self.rev_id)
+
+
+    def _remove_deleted(self):
+        """Remove deleted files from the working and stored inventories."""
+        for path, id, kind in self.delta.removed:
+            if self.work_inv.has_id(id):
+                del self.work_inv[id]
+            if self.new_inv.has_id(id):
+                del self.new_inv[id]
+
+
+    def _store_texts(self):
+        """Store new texts of modified/added files."""
+        for path, id, kind in self.delta.modified:
+            if kind != 'file':
+                continue
+            self._store_file_text(path, id)
+
+        for path, id, kind in self.delta.added:
+            if kind != 'file':
+                continue
+            self._store_file_text(path, id)
+
+        for old_path, new_path, id, kind, text_modified in self.delta.renamed:
+            if kind != 'file':
+                continue
+            if not text_modified:
+                continue
+            self._store_file_text(path, id)
+
+
+    def _store_file_text(self, path, id):
+        """Store updated text for one modified or added file."""
+        # TODO: Add or update the inventory entry for this file;
+        # put in the new text version
+        note('store new text for {%s} in revision {%s}', id, self.rev_id)
+        new_lines = self.work_tree.get_file(id).readlines()
+        weave_fn = self.branch.controlfilename(['weaves', id+'.weave'])
+        if os.path.exists(weave_fn):
+            w = read_weave(file(weave_fn, 'rb'))
+        else:
+            w = Weave()
+        w.add(self.rev_id, [], new_lines)
+        af = AtomicFile(weave_fn)
+        try:
+            write_weave_v5(w, af)
+            af.commit()
+        finally:
+            af.close()
+
+
+    def _gather(self):
+        """Build inventory preparatory to commit.
+
+        This adds any changed files into the text store, and sets their
+        test-id, sha and size in the returned inventory appropriately.
+
+        """
+        self.any_changes = False
+        self.new_inv = Inventory(self.work_inv.root.file_id)
+        self.missing_ids = []
+
+        for path, entry in self.work_inv.iter_entries():
+            ## TODO: Check that the file kind has not changed from the previous
+            ## revision of this file (if any).
+
+            p = self.branch.abspath(path)
+            file_id = entry.file_id
+            mutter('commit prep file %s, id %r ' % (p, file_id))
+
+            if (self.specific_files
+            and not is_inside_any(self.specific_files, path)):
+                mutter('  skipping file excluded from commit')
+                if self.basis_inv.has_id(file_id):
+                    # carry over with previous state
+                    self.new_inv.add(self.basis_inv[file_id].copy())
+                else:
+                    # omit this from committed inventory
+                    pass
+                continue
+
+            if not self.work_tree.has_id(file_id):
+                mutter("    file is missing, removing from inventory")
+                self.missing_ids.append(file_id)
+                continue
+
+            # this is present in the new inventory; may be new, modified or
+            # unchanged.
+            old_ie = self.basis_inv.has_id(file_id) and self.basis_inv[file_id]
+
+            entry = entry.copy()
+            self.new_inv.add(entry)
+
+            if old_ie:
+                old_kind = old_ie.kind
+                if old_kind != entry.kind:
+                    raise BzrError("entry %r changed kind from %r to %r"
+                            % (file_id, old_kind, entry.kind))
+
+            if entry.kind == 'directory':
+                if not isdir(p):
+                    raise BzrError("%s is entered as directory but not a directory"
+                                   % quotefn(p))
+            elif entry.kind == 'file':
+                if not isfile(p):
+                    raise BzrError("%s is entered as file but is not a file" % quotefn(p))
+
+                new_sha1 = self.work_tree.get_file_sha1(file_id)
+
+                if (old_ie
+                    and old_ie.text_sha1 == new_sha1):
+                    ## assert content == basis.get_file(file_id).read()
+                    entry.text_id = old_ie.text_id
+                    entry.text_sha1 = new_sha1
+                    entry.text_size = old_ie.text_size
+                    mutter('    unchanged from previous text_id {%s}' %
+                           entry.text_id)
+                else:
+                    content = file(p, 'rb').read()
+
+                    # calculate the sha again, just in case the file contents
+                    # changed since we updated the cache
+                    entry.text_sha1 = sha_string(content)
+                    entry.text_size = len(content)
+
+                    entry.text_id = gen_file_id(entry.name)
+                    self.branch.text_store.add(content, entry.text_id)
+                    mutter('    stored with text_id {%s}' % entry.text_id)
+
             marked = path + kind_marker(entry.kind)
             if not old_ie:
-                print 'added', marked
-                any_changes = True
+                self.reporter.added(marked)
+                self.any_changes = True
             elif old_ie == entry:
                 pass                    # unchanged
             elif (old_ie.name == entry.name
                   and old_ie.parent_id == entry.parent_id):
-                print 'modified', marked
-                any_changes = True
+                self.reporter.modified(marked)
+                self.any_changes = True
             else:
-                print 'renamed', marked
-                any_changes = True
-                        
-    return missing_ids, inv, any_changes
+                old_path = old_inv.id2path(file_id) + kind_marker(entry.kind)
+                self.reporter.renamed(old_path, marked)
+                self.any_changes = True
+
+
+
+def _gen_revision_id(branch, when):
+    """Return new revision-id."""
+    s = '%s-%s-' % (user_email(branch), compact_date(when))
+    s += hexlify(rand_bytes(8))
+    return s
 
 

=== modified file 'bzrlib/delta.py'
--- bzrlib/delta.py	2005-08-25 05:58:05 +0000
+++ bzrlib/delta.py	2005-09-07 04:00:21 +0000
@@ -69,9 +69,10 @@
             self.modified, self.unchanged)
 
     def has_changed(self):
-        changes = len(self.added) + len(self.removed) + len(self.renamed)
-        changes += len(self.modified) 
-        return (changes != 0)
+        return bool(self.modified
+                    or self.added
+                    or self.removed
+                    or self.renamed)
 
     def touches_file_id(self, file_id):
         """Return True if file_id is modified by this delta."""

=== modified file 'bzrlib/inventory.py'
--- bzrlib/inventory.py	2005-09-06 07:26:13 +0000
+++ bzrlib/inventory.py	2005-09-07 04:00:21 +0000
@@ -149,7 +149,8 @@
 
     def copy(self):
         other = InventoryEntry(self.file_id, self.name, self.kind,
-                               self.parent_id, text_id=self.text_id)
+                               self.parent_id)
+        other.text_id = self.text_id
         other.text_sha1 = self.text_sha1
         other.text_size = self.text_size
         # note that children are *not* copied; they're pulled across when
@@ -262,6 +263,15 @@
         self._byid = {self.root.file_id: self.root}
 
 
+    def copy(self):
+        other = Inventory(self.root.file_id)
+        for entry in self._byid.itervalues():
+            if entry == self.root:
+                continue
+            other.add(entry.copy())
+        return other
+
+
     def __iter__(self):
         return iter(self._byid)
 

=== modified file 'bzrlib/remotebranch.py'
--- bzrlib/remotebranch.py	2005-09-06 07:54:19 +0000
+++ bzrlib/remotebranch.py	2005-09-07 04:00:21 +0000
@@ -28,9 +28,10 @@
 from cStringIO import StringIO
 import urllib2
 
-from errors import BzrError, BzrCheckError
-from branch import Branch, BZR_BRANCH_FORMAT_5
-from trace import mutter
+from bzrlib.errors import BzrError, BzrCheckError
+from bzrlib.branch import Branch, BZR_BRANCH_FORMAT_5
+from bzrlib.trace import mutter
+from bzrlib.xml5 import serializer_v5
 
 # velocitynet.com.au transparently proxies connections and thereby
 # breaks keep-alive -- sucks!
@@ -158,7 +159,7 @@
             revf = self.revision_store[revision_id]
         except KeyError:
             raise NoSuchRevision(self, revision_id)
-        r = serializer_v4.read_revision(revf)
+        r = serializer_v5.read_revision(revf)
         if r.revision_id != revision_id:
             raise BzrCheckError('revision stored as {%s} actually contains {%s}'
                                 % (revision_id, r.revision_id))

=== modified file 'bzrlib/store.py'
--- bzrlib/store.py	2005-09-04 02:59:56 +0000
+++ bzrlib/store.py	2005-09-07 04:00:21 +0000
@@ -68,10 +68,12 @@
     def __init__(self, basedir):
         self._basedir = basedir
 
-    def _path(self, id):
-        if '\\' in id or '/' in id:
-            raise ValueError("invalid store id %r" % id)
-        return os.path.join(self._basedir, id)
+    def _path(self, entry_id):
+        if not isinstance(entry_id, basestring):
+            raise TypeError(type(entry_id))
+        if '\\' in entry_id or '/' in entry_id:
+            raise ValueError("invalid store id %r" % entry_id)
+        return os.path.join(self._basedir, entry_id)
 
     def __repr__(self):
         return "%s(%r)" % (self.__class__.__name__, self._basedir)

=== modified file 'bzrlib/xml.py'
--- bzrlib/xml.py	2005-09-06 06:44:11 +0000
+++ bzrlib/xml.py	2005-09-07 04:00:21 +0000
@@ -1,5 +1,4 @@
 #! /usr/bin/env python
-# -*- coding: UTF-8 -*-
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -128,9 +127,9 @@
             parent_id = ROOT_ID
 
         ie = InventoryEntry(elt.get('file_id'),
-                              elt.get('name'),
-                              elt.get('kind'),
-                              parent_id)
+                            elt.get('name'),
+                            elt.get('kind'),
+                            parent_id)
         ie.text_id = elt.get('text_id')
         ie.text_sha1 = elt.get('text_sha1')
 
@@ -219,145 +218,7 @@
 
 
 
-class _Serializer_v5(Serializer):
-    """Version 5 serializer
-
-    Packs objects into XML and vice versa.
-
-    You should use the serialzer_v5 singleton."""
-    
-    __slots__ = []
-    
-    def _pack_inventory(self, inv):
-        """Convert to XML Element"""
-        e = Element('inventory')
-        e.text = '\n'
-        if inv.root.file_id not in (None, ROOT_ID):
-            e.set('file_id', inv.root.file_id)
-        for path, ie in inv.iter_entries():
-            e.append(self._pack_entry(ie))
-        return e
-
-
-    def _pack_entry(self, ie):
-        """Convert InventoryEntry to XML element"""
-        assert ie.kind == 'directory' or ie.kind == 'file'
-        e = Element(ie.kind)
-        e.set('name', ie.name)
-        e.set('file_id', ie.file_id)
-
-        if ie.text_size != None:
-            e.set('text_size', '%d' % ie.text_size)
-
-        for f in ['text_version', 'text_sha1', 'entry_version']:
-            v = getattr(ie, f)
-            if v != None:
-                e.set(f, v)
-
-        # to be conservative, we don't externalize the root pointers
-        # for now, leaving them as null in the xml form.  in a future
-        # version it will be implied by nested elements.
-        if ie.parent_id != ROOT_ID:
-            assert isinstance(ie.parent_id, basestring)
-            e.set('parent_id', ie.parent_id)
-
-        e.tail = '\n'
-
-        return e
-
-
-    def _pack_revision(self, rev):
-        """Revision object -> xml tree"""
-        root = Element('revision',
-                       committer = rev.committer,
-                       timestamp = '%.9f' % rev.timestamp,
-                       revision_id = rev.revision_id,
-                       inventory_id = rev.inventory_id,
-                       inventory_sha1 = rev.inventory_sha1,
-                       )
-        if rev.timezone:
-            root.set('timezone', str(rev.timezone))
-        root.text = '\n'
-
-        msg = SubElement(root, 'message')
-        msg.text = rev.message
-        msg.tail = '\n'
-
-        if rev.parents:
-            pelts = SubElement(root, 'parents')
-            pelts.tail = pelts.text = '\n'
-            for rr in rev.parents:
-                assert isinstance(rr, RevisionReference)
-                p = SubElement(pelts, 'revision_ref')
-                p.tail = '\n'
-                assert rr.revision_id
-                p.set('revision_id', rr.revision_id)
-
-        return root
-
-    
-
-    def _unpack_inventory(self, elt):
-        """Construct from XML Element
-        """
-        assert elt.tag == 'inventory'
-        root_id = elt.get('file_id') or ROOT_ID
-        inv = Inventory(root_id)
-        for e in elt:
-            ie = self._unpack_entry(e)
-            if ie.parent_id == ROOT_ID:
-                ie.parent_id = root_id
-            inv.add(ie)
-        return inv
-
-
-    def _unpack_entry(self, elt):
-        kind = elt.tag
-        assert kind == 'directory' or kind == 'file'
-
-        parent_id = elt.get('parent_id')
-        if parent_id == None:
-            parent_id = ROOT_ID
-
-        ie = InventoryEntry(elt.get('file_id'),
-                            elt.get('name'),
-                            kind,
-                            parent_id)
-        ie.text_version = elt.get('text_version')
-        ie.entry_version = elt.get('entry_version')
-        ie.text_sha1 = elt.get('text_sha1')
-        v = elt.get('text_size')
-        ie.text_size = v and int(v)
-
-        return ie
-
-
-    def _unpack_revision(self, elt):
-        """XML Element -> Revision object"""
-        assert elt.tag == 'revision'
-        
-        rev = Revision(committer = elt.get('committer'),
-                       timestamp = float(elt.get('timestamp')),
-                       revision_id = elt.get('revision_id'),
-                       inventory_id = elt.get('inventory_id'),
-                       inventory_sha1 = elt.get('inventory_sha1')
-                       )
-
-        for p in elt.find('parents'):
-            assert p.tag == 'revision_ref', \
-                   "bad parent node tag %r" % p.tag
-            rev_ref = RevisionReference(p.get('revision_id'))
-            rev.parents.append(rev_ref)
-
-        v = elt.get('timezone')
-        rev.timezone = v and int(v)
-
-        rev.message = elt.findtext('message') # text of <message>
-        return rev
-
-
 
 """singleton instance"""
 serializer_v4 = _Serializer_v4()
 
-serializer_v5 = _Serializer_v5()

=== added file 'bzrlib/xml5.py'
--- bzrlib/xml5.py	1970-01-01 00:00:00 +0000
+++ bzrlib/xml5.py	2005-09-07 04:00:21 +0000
@@ -0,0 +1,163 @@
+#! /usr/bin/env python
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+from bzrlib.xml import ElementTree, SubElement, Element, Serializer
+from bzrlib.inventory import ROOT_ID, Inventory, InventoryEntry
+from bzrlib.revision import Revision, RevisionReference        
+from bzrlib.errors import BzrError
+
+
+
+
+
+class Serializer_v5(Serializer):
+    """Version 5 serializer
+
+    Packs objects into XML and vice versa.
+    """
+    
+    __slots__ = []
+    
+    def _pack_inventory(self, inv):
+        """Convert to XML Element"""
+        e = Element('inventory')
+        e.text = '\n'
+        if inv.root.file_id not in (None, ROOT_ID):
+            e.set('file_id', inv.root.file_id)
+        for path, ie in inv.iter_entries():
+            e.append(self._pack_entry(ie))
+        return e
+
+
+    def _pack_entry(self, ie):
+        """Convert InventoryEntry to XML element"""
+        assert ie.kind == 'directory' or ie.kind == 'file'
+        e = Element(ie.kind)
+        e.set('name', ie.name)
+        e.set('file_id', ie.file_id)
+
+        if ie.text_size != None:
+            e.set('text_size', '%d' % ie.text_size)
+
+        for f in ['text_version', 'text_sha1', 'entry_version']:
+            v = getattr(ie, f)
+            if v != None:
+                e.set(f, v)
+
+        # to be conservative, we don't externalize the root pointers
+        # for now, leaving them as null in the xml form.  in a future
+        # version it will be implied by nested elements.
+        if ie.parent_id != ROOT_ID:
+            assert isinstance(ie.parent_id, basestring)
+            e.set('parent_id', ie.parent_id)
+
+        e.tail = '\n'
+
+        return e
+
+
+    def _pack_revision(self, rev):
+        """Revision object -> xml tree"""
+        root = Element('revision',
+                       committer = rev.committer,
+                       timestamp = '%.9f' % rev.timestamp,
+                       revision_id = rev.revision_id,
+                       inventory_sha1 = rev.inventory_sha1,
+                       )
+        if rev.timezone:
+            root.set('timezone', str(rev.timezone))
+        root.text = '\n'
+
+        msg = SubElement(root, 'message')
+        msg.text = rev.message
+        msg.tail = '\n'
+
+        if rev.parents:
+            pelts = SubElement(root, 'parents')
+            pelts.tail = pelts.text = '\n'
+            for rr in rev.parents:
+                assert isinstance(rr, RevisionReference)
+                p = SubElement(pelts, 'revision_ref')
+                p.tail = '\n'
+                assert rr.revision_id
+                p.set('revision_id', rr.revision_id)
+
+        return root
+
+    
+
+    def _unpack_inventory(self, elt):
+        """Construct from XML Element
+        """
+        assert elt.tag == 'inventory'
+        root_id = elt.get('file_id') or ROOT_ID
+        inv = Inventory(root_id)
+        for e in elt:
+            ie = self._unpack_entry(e)
+            if ie.parent_id == ROOT_ID:
+                ie.parent_id = root_id
+            inv.add(ie)
+        return inv
+
+
+    def _unpack_entry(self, elt):
+        kind = elt.tag
+        assert kind == 'directory' or kind == 'file'
+
+        parent_id = elt.get('parent_id')
+        if parent_id == None:
+            parent_id = ROOT_ID
+
+        ie = InventoryEntry(elt.get('file_id'),
+                            elt.get('name'),
+                            kind,
+                            parent_id)
+        ie.text_version = elt.get('text_version')
+        ie.entry_version = elt.get('entry_version')
+        ie.text_sha1 = elt.get('text_sha1')
+        v = elt.get('text_size')
+        ie.text_size = v and int(v)
+
+        return ie
+
+
+    def _unpack_revision(self, elt):
+        """XML Element -> Revision object"""
+        assert elt.tag == 'revision'
+        
+        rev = Revision(committer = elt.get('committer'),
+                       timestamp = float(elt.get('timestamp')),
+                       revision_id = elt.get('revision_id'),
+                       inventory_sha1 = elt.get('inventory_sha1')
+                       )
+
+        parents = elt.find('parents') or []
+        for p in parents:
+            assert p.tag == 'revision_ref', \
+                   "bad parent node tag %r" % p.tag
+            rev_ref = RevisionReference(p.get('revision_id'))
+            rev.parents.append(rev_ref)
+
+        v = elt.get('timezone')
+        rev.timezone = v and int(v)
+
+        rev.message = elt.findtext('message') # text of <message>
+        return rev
+
+
+
+serializer_v5 = Serializer_v5()

